==> canada_post-0.0.5.tar.gz: lib/src/models/mailing_scenario/mailing_scenario.dart <==
import 'package:meta/meta.dart';
import 'package:xml/xml.dart';

import 'package:canada_post/src/models/parcel_characteristics/parcel_characteristics.dart';

import './destination.dart';
import './option.dart';
import './service.dart';

export './destination.dart';
export './option.dart';
export './service.dart';
export 'package:canada_post/src/models/parcel_characteristics/parcel_characteristics.dart';

final List<Option> defaultOptions = [
  Option(optionCode: "SO"),
  Option(optionCode: "COV", optionAmount: 100.0),
  Option(optionCode: "COD"),
  Option(optionCode: "PA18"),
  Option(optionCode: "PA19"),
  Option(optionCode: "HFP"),
  Option(optionCode: "DNS"),
  Option(optionCode: "LAD"),
];

class MailingScenario {
  final Destination destination;
  final ParcelCharacteristics parcelCharacteristics;
  
  /// Postal Code from which the parcel will be sent.
  /// Format ANANAN (only accepted with uppercase)
  final String originPostalCode;
  
  List<Service> services;
  
  List<Option> options;
  
  /// This element is required to obtain discounted rates for commercial customers 
  /// when the customer-number element is provided. 
  /// The contract-id element must be omitted for 
  /// Solutions for Small Business members or general businesss users.
  /// (numeric – up to 10 digits) 
  final String contractId;
  
  /// If you have a promotional discount code, enter it here. 
  /// The discount amount will be returned in the response under the adjustment structure.
  final String promoCode; 
  
  /// REQUIRED if quoteType is commercial (default).
  /// Otherwise, it must be null
  String customerNumber;

  /// Default: "commercial", otherwise: "counter"
  /// 
  /// Commercial is for small/medium/enterprise businesses;
  /// Counter is for customer shipments with no business account;
  String quoteType;
  
  /// The expected mailing date for the parcel.
  /// This date is used in calculations of the expected delivery date, however all rate quotes are based on the current system date.
  /// 
  /// (YYYY-MM-DD date format)
  String expectedMailingDate;

  MailingScenario(
      {@required this.destination,
      @required this.parcelCharacteristics,
      @required this.originPostalCode,
      this.options,
      this.services,
      this.contractId,
      this.promoCode,
      this.customerNumber,
      this.expectedMailingDate,
      this.quoteType = "commercial"}) {
    if (this.options.length > 20) {
      throw ArgumentError("incorrect number of options provided: min. 1, max. 20");
    }
    if (this.options == null) {
      this.options = defaultOptions;
    }

    if (this.customerNumber != null && this.quoteType == "counter") {
      throw ArgumentError(
          "can't provide a customerNumber for a quoteType \"counter\"");
    } else if (this.customerNumber == null && this.quoteType == "commercial") {
      throw ArgumentError('quoteType "commercial" requries a customerNumber');
    }

    if (this.expectedMailingDate == null) {
      this.expectedMailingDate =
          dateFormatted(DateTime.now().add(Duration(days: 1)));
    }

    if (this.quoteType != "commercial" && this.quoteType != "counter") {
      throw ArgumentError('quoteType must be "commercial" or "counter".');
    }
  }
}

String dateFormatted(DateTime date) {
  return date.year.toString() + '-' +
      (date.month < 10 ? '0' + date.month.toString() : date.month.toString()) + '-' +
      (date.day < 10 ? '0' + date.day.toString() : date.day.toString());
}

buildMailingScenario({XmlBuilder builder, MailingScenario mailingScenario}) {
  if (builder == null) builder =XmlBuilder();
  if (mailingScenario == null) throw ArgumentError("mailingScenario cannot be null");
  
  builder.element('mailing-scenario', nest: () {
    builder.namespace("http://www.canadapost.ca/ws/ship/rate-v3");

    buildOriginPostalCode(builder, mailingScenario);
    buildExpectedMailingDate(builder, mailingScenario);
    buildQuoteType(builder, mailingScenario);
    buildContractId(builder, mailingScenario);
    buildCustomerNumber(builder, mailingScenario);
    buildPromoCode(builder, mailingScenario);
    buildOptions(builder, mailingScenario.options);
    buildServices(builder, mailingScenario.services);
    buildDestination(builder, mailingScenario.destination);
    buildParcelCharacteristics(builder, mailingScenario.parcelCharacteristics);
  });
}

void buildPromoCode(XmlBuilder builder, MailingScenario mailingScenario) {
  if (mailingScenario.promoCode != null)
    builder.element('promo-code', nest: () {
      builder.text(mailingScenario.promoCode);
    });
}
void buildOriginPostalCode(XmlBuilder builder, MailingScenario mailingScenario) {
  builder.element('origin-postal-code', nest: () {
      builder.text(mailingScenario.originPostalCode);
  });
}
void buildExpectedMailingDate(XmlBuilder builder, MailingScenario mailingScenario) {
  builder.element('expected-mailing-date', nest: () {
    builder.text(mailingScenario.expectedMailingDate);
  });
}
void buildQuoteType(XmlBuilder builder, MailingScenario mailingScenario) {
  builder.element('quote-type', nest: () {
    builder.text(mailingScenario.quoteType);
  });
}
void buildContractId(XmlBuilder builder, MailingScenario mailingScenario) {
  if (mailingScenario.contractId != null)
    builder.element('contract-id', nest: () {
      builder.text(mailingScenario.contractId);
    });
}  
void buildCustomerNumber(XmlBuilder builder, MailingScenario mailingScenario) {
  if (mailingScenario.customerNumber != null)
    builder.element('customer-number', nest: () {
      builder.text(mailingScenario.customerNumber);
    });
}

==> canada_post-0.0.7.tar.gz: lib/src/models/mailing_scenario/mailing_scenario.dart <==
import 'package:meta/meta.dart';
import 'package:xml/xml.dart';

import 'package:canada_post/src/models/parcel_characteristics/parcel_characteristics.dart';

import './destination.dart';
import './option.dart';
import './service.dart';

export './destination.dart';
export './option.dart';
export './service.dart';
export 'package:canada_post/src/models/parcel_characteristics/parcel_characteristics.dart';

final List<Option> defaultOptions = [
  Option(optionCode: "SO"),
  Option(optionCode: "COV", optionAmount: 100.0),
  Option(optionCode: "COD"),
  Option(optionCode: "PA18"),
  Option(optionCode: "PA19"),
  Option(optionCode: "HFP"),
  Option(optionCode: "DNS"),
  Option(optionCode: "LAD"),
];

class MailingScenario {
  final Destination destination;
  final ParcelCharacteristics parcelCharacteristics;
  
  /// Postal Code from which the parcel will be sent.
  /// Format ANANAN (only accepted with uppercase)
  final String originPostalCode;
  
  List<Service> services;
  
  List<Option> options;
  
  /// This element is required to obtain discounted rates for commercial customers 
  /// when the customer-number element is provided. 
  /// The contract-id element must be omitted for 
  /// Solutions for Small Business members or general businesss users.
  /// (numeric – up to 10 digits) 
  final String contractId;
  
  /// If you have a promotional discount code, enter it here. 
  /// The discount amount will be returned in the response under the adjustment structure.
  final String promoCode; 
  
  /// REQUIRED if quoteType is commercial (default).
  /// Otherwise, it must be null
  String customerNumber;

  /// Default: "commercial", otherwise: "counter"
  /// 
  /// Commercial is for small/medium/enterprise businesses;
  /// Counter is for customer shipments with no business account;
  String quoteType;
  
  /// The expected mailing date for the parcel.
  /// This date is used in calculations of the expected delivery date, however all rate quotes are based on the current system date.
  /// 
  /// (YYYY-MM-DD date format)
  String expectedMailingDate;

  MailingScenario(
      {@required this.destination,
      @required this.parcelCharacteristics,
      @required this.originPostalCode,
      this.options,
      this.services,
      this.contractId,
      this.promoCode,
      this.customerNumber,
      this.expectedMailingDate,
      this.quoteType = "commercial"}) {
    if (this.options != null && this.options.length > 20) {
      throw ArgumentError("incorrect number of options provided: min. 1, max. 20");
    }
    if (this.options == null) {
      this.options = defaultOptions;
    }

    if (this.customerNumber != null && this.quoteType == "counter") {
      throw ArgumentError(
          "can't provide a customerNumber for a quoteType \"counter\"");
    } else if (this.customerNumber == null && this.quoteType == "commercial") {
      throw ArgumentError('quoteType "commercial" requries a customerNumber');
    }

    if (this.expectedMailingDate == null) {
      this.expectedMailingDate =
          dateFormatted(DateTime.now().add(Duration(days: 1)));
    }

    if (this.quoteType != "commercial" && this.quoteType != "counter") {
      throw ArgumentError('quoteType must be "commercial" or "counter".');
    }
  }
}

String dateFormatted(DateTime date) {
  return date.year.toString() + '-' +
      (date.month < 10 ? '0' + date.month.toString() : date.month.toString()) + '-' +
      (date.day < 10 ? '0' + date.day.toString() : date.day.toString());
}

XmlBuilder buildMailingScenario({XmlBuilder builder, MailingScenario mailingScenario}) {
  if (builder == null) builder = XmlBuilder();
  if (mailingScenario == null) throw ArgumentError("mailingScenario cannot be null");
  
  builder.element('mailing-scenario', nest: () {
    builder.namespace("http://www.canadapost.ca/ws/ship/rate-v3");

    buildOriginPostalCode(builder, mailingScenario);
    buildExpectedMailingDate(builder, mailingScenario);
    buildQuoteType(builder, mailingScenario);
    buildContractId(builder, mailingScenario);
    buildCustomerNumber(builder, mailingScenario);
    buildPromoCode(builder, mailingScenario);
    buildOptions(builder, mailingScenario.options);
    buildServices(builder, mailingScenario.services);
    buildDestination(builder, mailingScenario.destination);
    buildParcelCharacteristics(builder, mailingScenario.parcelCharacteristics);
  });

  return builder;
}

void buildPromoCode(XmlBuilder builder, MailingScenario mailingScenario) {
  if (mailingScenario.promoCode != null)
    builder.element('promo-code', nest: () {
      builder.text(mailingScenario.promoCode);
    });
}
void buildOriginPostalCode(XmlBuilder builder, MailingScenario mailingScenario) {
  builder.element('origin-postal-code', nest: () {
      builder.text(mailingScenario.originPostalCode);
  });
}
void buildExpectedMailingDate(XmlBuilder builder, MailingScenario mailingScenario) {
  builder.element('expected-mailing-date', nest: () {
    builder.text(mailingScenario.expectedMailingDate);
  });
}
void buildQuoteType(XmlBuilder builder, MailingScenario mailingScenario) {
  builder.element('quote-type', nest: () {
    builder.text(mailingScenario.quoteType);
  });
}
void buildContractId(XmlBuilder builder, MailingScenario mailingScenario) {
  if (mailingScenario.contractId != null)
    builder.element('contract-id', nest: () {
      builder.text(mailingScenario.contractId);
    });
}  
void buildCustomerNumber(XmlBuilder builder, MailingScenario mailingScenario) {
  if (mailingScenario.customerNumber != null)
    builder.element('customer-number', nest: () {
      builder.text(mailingScenario.customerNumber);
    });
}
